###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from common import *
from common_dsp import *
from mem_init_file import list_to_hex

import matplotlib.pyplot as plt
import numpy as np
from scipy.fftpack import fft,ifft, fftfreq, fftshift

# Purpose:
# . Generate the NOF_INPUTS HEX files for the block generators that feed the WPFB
# Description:
# . 

NOF_INPUTS = 12
COMPLEX_WIDTH = 6
NOF_WORDS_PER_BLOCK = 64
MEM_WIDTH  = COMPLEX_WIDTH*2
MEM_DEPTH  = 2*NOF_WORDS_PER_BLOCK # We're interleaving, hence twice the depth

PATH = "../hex"
FILENAME = "composite_signals"

###############################################################################
# Create a 120MHz sine wave
###############################################################################
N = NOF_WORDS_PER_BLOCK
t=range(N)

# sample spacing
T = 1.0 / N

x = np.linspace(0.0, N*T, N)

AMPL = 31 # 5 bits + sign available
# my singal
#s0 = 7 * np.exp( 1 * 1.j * 2.0*np.pi*x)
#s1 = 7 * np.exp( 5 * 1.j * 2.0*np.pi*x)
#s2 = 7 * np.exp( 10 * 1.j * 2.0*np.pi*x)
#s3 = 7 * np.exp( 15 * 1.j * 2.0*np.pi*x)
s4 =  31 * np.exp( 8 * 1.j * 2.0*np.pi*x)

s = s4 #  +s1 + s2 + s3 + s4
################################################################################
## Plot the signal
################################################################################
plt.plot(t, s.real, 'b-', t, s.imag, 'r--')
plt.legend(('real', 'imaginary'))
plt.show()
#
################################################################################
## Plot FFT
################################################################################
yf = fft(s)
xf = fftfreq(N, T)
xf = fftshift(xf)
yplot = fftshift(yf)
plt.plot(xf, 1.0/N * np.abs(yplot))
plt.grid()
plt.show()

###############################################################################
# Convert complex floats to concatenated integers
###############################################################################
concat_list = concat_complex(s, COMPLEX_WIDTH)

###############################################################################
# Interleave 2 lists into one
###############################################################################
inter_list = interleave([concat_list,concat_list])

###############################################################################
# Use this list for each block generator
###############################################################################
bg_lists = NOF_INPUTS*[inter_list]

###############################################################################
# Write the HEX files
############################################################################### 
for input_nr in range(NOF_INPUTS):
    list_to_hex( bg_lists[input_nr], PATH+"/"+FILENAME+'_'+str(input_nr)+".hex", MEM_WIDTH, MEM_DEPTH)

