
from common import *
import os
import matplotlib.pyplot as plt
import numpy as np
from scipy.fftpack import fft,ifft, fftfreq, fftshift

REC_FILE = os.environ['RADIOHDL']+'/applications/apertif_unb1_correlator/tb/rec/wpfb_src_out_arr0.rec'

WPFB_OUT_DATA_WIDTH = 12

################################################################################
# Read the lines from the file and turn them into a list
################################################################################
with open (REC_FILE, "r") as recfile:
    lines = recfile.read().splitlines()

################################################################################
# Extract the complex fields; create new complex list
################################################################################
complex_list = []
raw_data_int = []
raw_data_str = []

for line_nr,line in enumerate(lines):
#    line = line.replace('X', '0') # Get rid of the 'X' on bit 24 (we only use 0..23)
    split_line = line.split(' ')

#    print split_line
    str_re = split_line[3]
    str_im = split_line[4]
    raw_data_str.append( [str_re, str_im] )

    re = to_signed(int(str_re, 32), WPFB_OUT_DATA_WIDTH)
    im = to_signed(int(str_im, 32), WPFB_OUT_DATA_WIDTH) 
    raw_data_int.append( [re, im] )
    complex_word = complex(re,im)
    complex_list.append( complex_word )

################################################################################
# Split the list into chunks of 128 (note SOP/EOP tags of WPFB output are
# incorrect:64 samples/block instead of 128).
################################################################################
complex_list_128 = split_list(complex_list, 128)
f = deinterleave(complex_list_128[5], 2)[1]

#complex_list_64 = split_list(complex_list, 64)
#f = complex_list_64[6]

################################################################################
## Plot FFT
################################################################################
N=64
T = 1.0 / N

yf = fft(f)
xf = fftfreq(N, T)
xf = fftshift(xf)
yplot = fftshift(yf)
plt.plot(xf, 1.0/N * np.abs(yplot))
plt.grid()
plt.show()

