-------------------------------------------------------------------------------
--
-- Copyright (C) 2009
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb_common_lib, diagnostics_lib, dp_lib, tr_xaui_lib, tr_10GbE_lib, technology_lib, tech_memory_lib;
use unb_common_lib.unb_common_pkg.all;
USE IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE unb_common_lib.unb_common_pkg.ALL;
--USE tr_xaui_lib.tr_xaui_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE technology_lib.technology_pkg.ALL;


ENTITY unb2_test IS
  port (

    -- GENERAL
    CLK                    : IN    STD_LOGIC; -- External system clock
    PPS                    : IN    STD_LOGIC; -- External system sync
    WDI                    : OUT   STD_LOGIC; -- Watchdog Clear
    INTA                   : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB                   : INOUT STD_LOGIC; -- FPGA interconnect line

    -- 1GbE Control Interfaces
    ETH_CLK                : in std_logic;
    ETH_SGIN               : in std_logic_vector (1 downto 0);
    ETH_SGOUT              : out std_logic_vector (1 downto 0);

    -- Transceiver clocks
    SA_CLK                 : IN    STD_LOGIC; -- SerDes reference clock front 
    SB_CLK                 : IN    STD_LOGIC; -- SerDes reference clock back
    BCK_REF_CLK            : IN    STD_LOGIC; -- SerDes reference clock back

    -- SO-DIMM DDR4 Memory Bank i2c (common)
    MB_SCL                 : inout std_logic;
    MB_SDA                 : inout std_logic;
    -- SO-DIMM DDR4 Memory Bank I
    MB_I_RZQ       : in   STD_LOGIC;
    MB_I_REF_CLK   : in    STD_LOGIC; -- External reference clock
    MB_I_A : out std_logic_vector (13 downto 0);
    MB_I_ACT_N : out std_logic_vector(0 downto 0);
    MB_I_BA : out std_logic_vector (1 downto 0);
    MB_I_BG : out std_logic_vector (1 downto 0);
    MB_I_CAS_A15 : out std_logic;
    MB_I_CB : inout std_logic_vector (7 downto 0);
    MB_I_CK : out std_logic_vector (1 downto 0);
    MB_I_CK_n : out std_logic_vector (1 downto 0);
    MB_I_CKE : out std_logic_vector (1 downto 0);
    MB_I_CS : out std_logic_vector (1 downto 0);
    MB_I_DM : inout std_logic_vector (8 downto 0); -- !! temporarily chg to inout
    MB_I_DQ : inout std_logic_vector (63 downto 0);
    MB_I_DQS : inout std_logic_vector (8 downto 0);
    MB_I_DQS_n : inout std_logic_vector (8 downto 0);
    MB_I_ODT : out std_logic_vector (1 downto 0);
    MB_I_PARITY : out std_logic_vector(0 downto 0);
    MB_I_RAS_A16 : out std_logic;
    MB_I_WE_A14 : out std_logic;
    MB_I_RESET_N           : out std_logic_vector(0 downto 0);
    MB_I_ALERT_N           : in std_logic_vector(0 downto 0);
    -- SO-DIMM DDR4 Memory Bank II
    MB_II_RZQ    : in   STD_LOGIC;
    MB_II_REF_CLK : in    STD_LOGIC; -- External reference clock
    MB_II_A : out std_logic_vector (13 downto 0);
    MB_II_ACT_N : out std_logic_vector(0 downto 0);
    MB_II_BA : out std_logic_vector (1 downto 0);
    MB_II_BG : out std_logic_vector (1 downto 0);
    MB_II_CAS_A15 : out std_logic;
    MB_II_CB : inout std_logic_vector (7 downto 0);
    MB_II_CK : out std_logic_vector (1 downto 0);
    MB_II_CK_n : out std_logic_vector (1 downto 0);
    MB_II_CKE : out std_logic_vector (1 downto 0);
    MB_II_CS : out std_logic_vector (1 downto 0);
    MB_II_DM : inout std_logic_vector (8 downto 0); -- !! temporarily chg to inout
    MB_II_DQ : inout std_logic_vector (63 downto 0);
    MB_II_DQS : inout std_logic_vector (8 downto 0);
    MB_II_DQS_n : inout std_logic_vector (8 downto 0);
    MB_II_ODT : out std_logic_vector (1 downto 0);
    MB_II_PARITY : out std_logic_vector(0 downto 0);
    MB_II_RAS_A16 : out std_logic;
    MB_II_WE_A14 : out std_logic;
    MB_II_RESET_N          : out std_logic_vector(0 downto 0);
    MB_II_ALERT_N           : in std_logic_vector(0 downto 0);
    
    -- back transceivers
    BCK_RX : in std_logic_vector (47 downto 0);
    BCK_TX : out std_logic_vector (47 downto 0);
    BCK_SDA : inout std_logic_vector (2 downto 0);
    BCK_SCL : inout std_logic_vector (2 downto 0);
    BCK_ERR : inout std_logic_vector (2 downto 0);
    -- ring transceivers
    RING_0_RX : in std_logic_vector (11 downto 0);
    RING_0_TX : out std_logic_vector (11 downto 0);
    RING_1_RX : in std_logic_vector (11 downto 0);
    RING_1_TX : out std_logic_vector (11 downto 0);
    -- pmbus
    PMBUS_SC : inout std_logic;
    PMBUS_SD : inout std_logic;
    PMBUS_ALERT : in std_logic;
    -- front transceivers
    QSFP_0_RX : in std_logic_vector (3 downto 0);
    QSFP_0_TX : out std_logic_vector (3 downto 0);
    QSFP_1_RX : in std_logic_vector (3 downto 0);
    QSFP_1_TX : out std_logic_vector (3 downto 0);
    QSFP_2_RX : in std_logic_vector (3 downto 0);
    QSFP_2_TX : out std_logic_vector (3 downto 0);
    QSFP_3_RX : in std_logic_vector (3 downto 0);
    QSFP_3_TX : out std_logic_vector (3 downto 0);
    QSFP_4_RX : in std_logic_vector (3 downto 0);
    QSFP_4_TX : out std_logic_vector (3 downto 0);
    QSFP_5_RX : in std_logic_vector (3 downto 0);
    QSFP_5_TX : out std_logic_vector (3 downto 0);
    QSFP_SDA : inout std_logic_vector (5 downto 0);
    QSFP_SCL : inout std_logic_vector (5 downto 0);
    QSFP_RST : inout std_logic;
    -- I2C Interface to Sensors
    SENS_SC                : inOUT   STD_LOGIC;
    SENS_SD                : INOUT STD_LOGIC;
     -- Others
--    CFG_DATA               : inout std_logic_vector (3 downto 0);
    VERSION                : IN    STD_LOGIC_VECTOR(1 DOWNTO 0);
    ID                     : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);
    TESTIO                 : INOUT STD_LOGIC_VECTOR(7 DOWNTO 0)
  );

end unb2_test;


architecture str of unb2_test is

    component ddr4 is
      port (
        global_reset_n      : in    std_logic                      := 'X';             -- reset_n
        pll_ref_clk         : in    std_logic                      := 'X';             -- clk
        oct_rzqin           : in    std_logic                      := 'X';             -- oct_rzqin
        mem_ck              : out   std_logic_vector(1 downto 0);                      -- mem_ck
        mem_ck_n            : out   std_logic_vector(1 downto 0);                      -- mem_ck_n
        mem_a               : out   std_logic_vector(16 downto 0);                     -- mem_a
        mem_act_n           : out   std_logic_vector(0 downto 0);                      -- mem_act_n
        mem_ba              : out   std_logic_vector(1 downto 0);                      -- mem_ba
        mem_bg              : out   std_logic_vector(1 downto 0);                      -- mem_bg
        mem_cke             : out   std_logic_vector(1 downto 0);                      -- mem_cke
        mem_cs_n            : out   std_logic_vector(1 downto 0);                      -- mem_cs_n
        mem_odt             : out   std_logic_vector(1 downto 0);                      -- mem_odt
        mem_reset_n         : out   std_logic_vector(0 downto 0);                      -- mem_reset_n
        mem_alert_n         : in   std_logic_vector(0 downto 0);                       -- mem_alert_n
        mem_par             : out   std_logic_vector(0 downto 0);                      -- mem_par  ** new in 14.0 **
        mem_dqs             : inout std_logic_vector(8 downto 0)   := (others => 'X'); -- mem_dqs
        mem_dqs_n           : inout std_logic_vector(8 downto 0)   := (others => 'X'); -- mem_dqs_n
        mem_dq              : inout std_logic_vector(71 downto 0)  := (others => 'X'); -- mem_dq
        mem_dbi_n           : inout std_logic_vector(8 downto 0)   := (others => 'X'); -- mem_dbi_n
        local_cal_success   : out   std_logic;                                         -- local_cal_success
        local_cal_fail      : out   std_logic;                                         -- local_cal_fail
        emif_usr_reset_n    : out   std_logic;                                         -- reset_n
        emif_usr_clk        : out   std_logic;                                         -- clk
        amm_ready_0         : out   std_logic;                                         -- waitrequest_n
        amm_read_0          : in    std_logic                      := 'X';             -- read
        amm_write_0         : in    std_logic                      := 'X';             -- write
        amm_address_0       : in    std_logic_vector(26 downto 0)  := (others => 'X'); -- address ** chg from 23 bits in 14.0 **
        amm_readdata_0      : out   std_logic_vector(575 downto 0);                    -- readdata
        amm_writedata_0     : in    std_logic_vector(575 downto 0) := (others => 'X'); -- writedata
        amm_burstcount_0    : in    std_logic_vector(6 downto 0)   := (others => 'X'); -- burstcount ** chg from 8 bits in 14.0 **
        amm_byteenable_0    : in    std_logic_vector(71 downto 0)  := (others => 'X'); -- byteenable
        amm_readdatavalid_0 : out   std_logic                                          -- readdatavalid
      );
    end component ddr4;


   component system_pll is
     port (
       refclk    : in  std_logic := 'X'; -- clk
       rst       : in  std_logic := 'X';
       locked    : out std_logic;       
       outclk_0  : out std_logic;        -- outclk0
       outclk_1  : out std_logic;        -- outclk1
       outclk_2  : out std_logic         -- outclk2
     );
  end component system_pll;

  component unb2_test_qsys is
    port (
      clk_clk                          : in    std_logic := 'X'; -- clk
      reset_reset_n                    : in    std_logic := 'X'; -- reset_n
      avs_i2c_master_0_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_0_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_0_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_0_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_1_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_1_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_1_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_1_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_11_i2c_sda_export : inout std_logic := 'X'; -- export
      avs_i2c_master_11_i2c_scl_export : inout std_logic := 'X'; -- export
      avs_i2c_master_11_sync_export    : in    std_logic := 'X'; -- export
      avs_i2c_master_11_gs_sim_export  : in    std_logic := 'X'; -- export
      avs_i2c_master_10_i2c_sda_export : inout std_logic := 'X'; -- export
      avs_i2c_master_10_i2c_scl_export : inout std_logic := 'X'; -- export
      avs_i2c_master_10_sync_export    : in    std_logic := 'X'; -- export
      avs_i2c_master_10_gs_sim_export  : in    std_logic := 'X'; -- export
      avs_i2c_master_9_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_9_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_9_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_9_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_8_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_8_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_8_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_8_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_7_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_7_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_7_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_7_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_6_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_6_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_6_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_6_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_5_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_5_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_5_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_5_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_4_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_4_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_4_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_4_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_3_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_3_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_3_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_3_i2c_sda_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_2_gs_sim_export   : in    std_logic := 'X'; -- export
      avs_i2c_master_2_sync_export     : in    std_logic := 'X'; -- export
      avs_i2c_master_2_i2c_scl_export  : inout std_logic := 'X'; -- export
      avs_i2c_master_2_i2c_sda_export  : inout std_logic := 'X'; -- export
      eth_tse_0_serial_connection_rxp_0          : in    std_logic := 'X'; -- rxp
      eth_tse_0_serial_connection_txp_0          : out   std_logic;        -- txp
      eth_tse_0_pcs_ref_clk_clock_connection_clk : in    std_logic := 'X'; -- clk
      eth_tse_1_pcs_ref_clk_clock_connection_clk : in    std_logic := 'X'; -- clk
      eth_tse_1_serial_connection_rxp_0          : in    std_logic := 'X'; -- rxp
      eth_tse_1_serial_connection_txp_0          : out   std_logic;         -- txp
      pio_0_external_connection_export           : in    std_logic_vector(10 downto 0) := (others => 'X');  -- export

               reg_mac_front_read_export                  : out   std_logic;                                        -- export
                reg_mac_front_address_export               : out   std_logic_vector(14 downto 0);                    -- export
                reg_mac_front_write_export                 : out   std_logic;                                        -- export
                reg_mac_front_writedata_export             : out   std_logic_vector(31 downto 0);                    -- export
                reg_mac_front_readdata_export              : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_mac_front_waitrequest_export           : in    std_logic                     := 'X';             -- export
                reg_mac_back_address_export                : out   std_logic_vector(14 downto 0);                    -- export
                reg_mac_back_write_export                  : out   std_logic;                                        -- export
                reg_mac_back_writedata_export              : out   std_logic_vector(31 downto 0);                    -- export
                reg_mac_back_read_export                   : out   std_logic;                                        -- export
                reg_mac_back_readdata_export               : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_mac_back_waitrequest_export            : in    std_logic                     := 'X';             -- export
                reg_hdr_insert_front_readdata_export       : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_hdr_insert_front_read_export           : out   std_logic;                                        -- export
                reg_hdr_insert_front_writedata_export      : out   std_logic_vector(31 downto 0);                    -- export
                reg_hdr_insert_front_address_export        : out   std_logic_vector(2 downto 0);                     -- export
                reg_hdr_insert_front_write_export          : out   std_logic;                                        -- export
                reg_hdr_insert_back_readdata_export        : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_hdr_insert_back_read_export            : out   std_logic;                                        -- export
                reg_hdr_insert_back_writedata_export       : out   std_logic_vector(31 downto 0);                    -- export
                reg_hdr_insert_back_write_export           : out   std_logic;                                        -- export
                reg_hdr_insert_back_address_export         : out   std_logic_vector(2 downto 0);                     -- export
                ram_hdr_insert_front_readdata_export       : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                ram_hdr_insert_front_read_export           : out   std_logic;                                        -- export
                ram_hdr_insert_front_writedata_export      : out   std_logic_vector(31 downto 0);                    -- export
                ram_hdr_insert_front_write_export          : out   std_logic;                                        -- export
                ram_hdr_insert_front_address_export        : out   std_logic_vector(5 downto 0);                     -- export
                ram_hdr_insert_back_address_export         : out   std_logic_vector(5 downto 0);                     -- export
                ram_hdr_insert_back_write_export           : out   std_logic;                                        -- export
                ram_hdr_insert_back_writedata_export       : out   std_logic_vector(31 downto 0);                    -- export
                ram_hdr_insert_back_read_export            : out   std_logic;                                        -- export
                ram_hdr_remove_front_readdata_export       : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                ram_hdr_remove_front_read_export           : out   std_logic;                                        -- export
                ram_hdr_remove_front_writedata_export      : out   std_logic_vector(31 downto 0);                    -- export
                ram_hdr_remove_front_write_export          : out   std_logic;                                        -- export
                ram_hdr_remove_front_address_export        : out   std_logic_vector(5 downto 0);                     -- export
                ram_hdr_remove_back_address_export         : out   std_logic_vector(5 downto 0);                     -- export
                ram_hdr_remove_back_write_export           : out   std_logic;                                        -- export
                ram_hdr_remove_back_writedata_export       : out   std_logic_vector(31 downto 0);                    -- export
                ram_hdr_remove_back_read_export            : out   std_logic;                                        -- export
                ram_hdr_remove_back_readdata_export        : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_front_0_address_export     : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_front_0_write_export       : out   std_logic;                                        -- export
                reg_diagnostics_front_0_writedata_export   : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_front_0_read_export        : out   std_logic;                                        -- export
                reg_diagnostics_front_0_readdata_export    : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_front_1_readdata_export    : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_front_1_read_export        : out   std_logic;                                        -- export
                reg_diagnostics_front_1_writedata_export   : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_front_1_write_export       : out   std_logic;                                        -- export
                reg_diagnostics_front_1_address_export     : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_front_2_readdata_export    : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_front_2_read_export        : out   std_logic;                                        -- export
                reg_diagnostics_front_2_writedata_export   : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_front_2_write_export       : out   std_logic;                                        -- export
                reg_diagnostics_front_2_address_export     : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_back_0_readdata_export     : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_back_0_read_export         : out   std_logic;                                        -- export
                reg_diagnostics_back_0_writedata_export    : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_back_0_write_export        : out   std_logic;                                        -- export
                reg_diagnostics_back_0_address_export      : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_back_1_address_export      : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_back_1_write_export        : out   std_logic;                                        -- export
                reg_diagnostics_back_1_writedata_export    : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_back_1_read_export         : out   std_logic;                                        -- export
                reg_diagnostics_back_1_readdata_export     : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_back_2_address_export      : out   std_logic_vector(5 downto 0);                     -- export
                reg_diagnostics_back_2_write_export        : out   std_logic;                                        -- export
                reg_diagnostics_back_2_writedata_export    : out   std_logic_vector(31 downto 0);                    -- export
                reg_diagnostics_back_2_readdata_export     : in    std_logic_vector(31 downto 0) := (others => 'X'); -- export
                reg_diagnostics_back_2_read_export         : out   std_logic                                         -- export

   );
  end component unb2_test_qsys;

    -- constants
    constant cs_sim                : std_logic := '0';
    constant cs_sync               : std_logic := '1';
    --CONSTANT c_block_len            : NATURAL := 180; -- = 1440 user bytes. Including packetizing: 1508 bytes.
    CONSTANT c_block_len            : NATURAL := 1118;-- = 8944 user bytes. Including packetizing: 9012 bytes.
    type t_mem_mosi_arr is array (0 to 2) of t_mem_mosi;
    type t_mem_miso_arr is array (0 to 2) of t_mem_miso;

    -- general reset and clock signals
    signal reset_n                 : std_logic := '0';
    signal reset_p                 : std_logic := '0';
    signal pout_wdi                : std_logic := '0';
    signal sys_clk                 : std_logic := '0';
    signal sys_locked              : std_logic := '0';
    signal mm_clk                  : std_logic := '0';
    signal clk_125                 : std_logic := '0';

    -- signals for the ddr4 controllers
    signal local_i_cal_success     : std_logic;
    signal local_i_cal_fail        : std_logic;
    signal local_i_reset_n         : std_logic;
    signal local_i_clk             : std_logic;
    signal local_i_ready           : std_logic;
    signal local_i_read            : std_logic;
    signal local_i_write           : std_logic;
    signal local_i_address         : std_logic_vector(26 downto 0);
    signal local_i_readdata        : std_logic_vector(575 downto 0);
    signal local_i_writedata       : std_logic_vector(575 downto 0);
    signal local_i_burstcount      : std_logic_vector(6 downto 0);
    signal local_i_be              : std_logic_vector(71 downto 0);
    signal local_i_read_data_valid : std_logic;
    signal mb_i_a_internal         : std_logic_vector(16 downto 0);
    signal local_ii_cal_success    : std_logic;
    signal local_ii_cal_fail       : std_logic;
    signal local_ii_reset_n        : std_logic;
    signal local_ii_clk            : std_logic;
    signal local_ii_ready          : std_logic;
    signal local_ii_read           : std_logic;
    signal local_ii_write          : std_logic;
    signal local_ii_address        : std_logic_vector(26 downto 0);
    signal local_ii_readdata       : std_logic_vector(575 downto 0);
    signal local_ii_writedata      : std_logic_vector(575 downto 0);
    signal local_ii_burstcount     : std_logic_vector(6 downto 0);
    signal local_ii_be             : std_logic_vector(71 downto 0);
    signal local_ii_read_data_valid: std_logic;
    signal mb_ii_a_internal        : std_logic_vector(16 downto 0);

    -- signals for the mm buses
    signal reg_diagnostics_front_mosi    : t_mem_mosi_arr;
    signal reg_diagnostics_front_miso    : t_mem_miso_arr;
    signal reg_hdr_insert_front_mosi     : t_mem_mosi;
    signal ram_hdr_insert_front_mosi     : t_mem_mosi;
    signal ram_hdr_remove_front_mosi     : t_mem_mosi;
    signal ram_hdr_remove_front_miso     : t_mem_miso;
    signal reg_mac_front_mosi            : t_mem_mosi;
    signal reg_mac_front_miso            : t_mem_miso;

    signal reg_diagnostics_back_mosi     : t_mem_mosi_arr;
    signal reg_diagnostics_back_miso     : t_mem_miso_arr;
    signal reg_hdr_insert_back_mosi      : t_mem_mosi;
    signal ram_hdr_insert_back_mosi      : t_mem_mosi;
    signal ram_hdr_remove_back_mosi      : t_mem_mosi;
    signal ram_hdr_remove_back_miso      : t_mem_miso;
    signal reg_mac_back_mosi             : t_mem_mosi;
    signal reg_mac_back_miso             : t_mem_miso;
--    signal xaui_rx_arr_dummy             : t_unb_xaui_sl_2arr(47 downto 0) := (others=>"0000");
    signal xaui_rx_arr_dummy             : t_xaui_arr(47 DOWNTO 0);

    -- signals for the transceivers
    signal tx_serial_data_front    : std_logic_vector(47 downto 0);
    signal rx_serial_data_front    : std_logic_vector(47 downto 0);
    signal diagnostics_front_snk_in_arr   : t_dp_sosi_arr(47 DOWNTO 0);
    signal diagnostics_front_snk_out_arr  : t_dp_siso_arr(47 DOWNTO 0);
    signal diagnostics_front_src_out_arr  : t_dp_sosi_arr(47 DOWNTO 0);
    signal diagnostics_front_src_in_arr   : t_dp_siso_arr(47 DOWNTO 0);


    signal tx_serial_data_back    : std_logic_vector(47 downto 0);
    signal rx_serial_data_back    : std_logic_vector(47 downto 0);
    signal diagnostics_back_snk_in_arr   : t_dp_sosi_arr(47 DOWNTO 0);
    signal diagnostics_back_snk_out_arr  : t_dp_siso_arr(47 DOWNTO 0);
    signal diagnostics_back_src_out_arr  : t_dp_sosi_arr(47 DOWNTO 0);
    signal diagnostics_back_src_in_arr   : t_dp_siso_arr(47 DOWNTO 0);

    -- signals for the bidirectional and misc ios
    signal inta_in    : std_logic;
    signal intb_in    : std_logic;
    signal testio_in  : std_logic_vector(7 downto 0);
    signal bck_err_in : std_logic_vector(2 downto 0);
    signal inta_out   : std_logic;
    signal intb_out   : std_logic;
    signal testio_out : std_logic_vector(7 downto 0);
    signal bck_err_out : std_logic_vector(2 downto 0);
    signal ver_id_pmbusalert     : std_logic_vector(10 downto 0);

begin

    WDI <= 'Z';

    --  ****** DDR4 memory controllers ******

    mb_i_a <= mb_i_a_internal(13 downto 0);
    mb_i_we_a14 <= mb_i_a_internal(14);
    mb_i_cas_a15 <= mb_i_a_internal(15);
    mb_i_ras_a16 <= mb_i_a_internal(16);

    local_i_proc : process(local_i_clk, local_i_reset_n)
    begin
      if local_i_reset_n = '0' then
        local_i_read <= '0';
        local_i_write <= '0';
        local_i_address <= (others => '0');
        local_i_writedata <= (others => '0');
        local_i_burstcount <= (others => '0');
        local_i_be <= (others => '0');
      else
        if local_i_clk'event and local_i_clk = '1' then
          local_i_be <= (others => '1');
          if local_i_ready = '1' then
	    local_i_read <= not local_i_read;
            local_i_write <= local_i_read_data_valid;
            local_i_address <= local_i_address+1;
	    if local_i_read_data_valid = '1' then
              local_i_writedata <= not local_i_readdata;
	    else
	      local_i_writedata <= (others => '1');
            end if;
          end if;
	end if;
      end if;
    end process;

    u_ddr4_i : ddr4 
      port map (
        global_reset_n      => reset_n,
        pll_ref_clk         => MB_I_REF_CLK,
        oct_rzqin           => MB_I_RZQ,
        mem_ck              => mb_i_ck,
        mem_ck_n            => mb_i_ck_n,
        mem_a               => mb_i_a_internal,
        mem_act_n           => mb_i_act_n,
        mem_ba              => mb_i_ba,
        mem_bg              => mb_i_bg,
        mem_cke             => mb_i_cke,
        mem_cs_n            => mb_i_cs, 
        mem_odt             => mb_i_odt,
        mem_reset_n         => mb_i_reset_n,
        mem_alert_n         => mb_i_alert_n,
        mem_par             => mb_i_parity,
        mem_dqs             => mb_i_dqs,
        mem_dqs_n           => mb_i_dqs_n,
        mem_dq(63 downto 0) => mb_i_dq,
        mem_dq(71 downto 64)=> mb_i_cb,
        mem_dbi_n           => mb_i_dm,
        local_cal_success   => local_i_cal_success,
        local_cal_fail      => local_i_cal_fail,
        emif_usr_reset_n    => local_i_reset_n,
        emif_usr_clk        => local_i_clk,
        amm_ready_0         => local_i_ready,
        amm_read_0          => local_i_read,
        amm_write_0         => local_i_write,
        amm_address_0       => local_i_address,
        amm_readdata_0      => local_i_readdata,
        amm_writedata_0     => local_i_writedata,
        amm_burstcount_0    => local_i_burstcount,
        amm_byteenable_0    => local_i_be,
        amm_readdatavalid_0 => local_i_read_data_valid
     );

    mb_ii_a <= mb_ii_a_internal(13 downto 0);
    mb_ii_we_a14 <= mb_ii_a_internal(14);
    mb_ii_cas_a15 <= mb_ii_a_internal(15);
    mb_ii_ras_a16 <= mb_ii_a_internal(16);

    local_ii_proc : process(local_ii_clk, local_ii_reset_n)
    begin
      if local_ii_reset_n = '0' then
        local_ii_read <= '0';
        local_ii_write <= '0';
        local_ii_address <= (others => '0');
        local_ii_writedata <= (others => '0');
        local_ii_burstcount <= (others => '0');
        local_ii_be <= (others => '0');
      else
        if local_ii_clk'event and local_ii_clk = '1' then
          local_ii_be <= (others => '1');
          if local_ii_ready = '1' then
	    local_ii_read <= not local_ii_read;
            local_ii_write <= local_ii_read_data_valid;
            local_ii_address <= local_ii_address+1;
	    if local_ii_read_data_valid = '1' then
              local_ii_writedata <= not local_ii_readdata;
	    else
	      local_ii_writedata <= (others => '1');
            end if;
          end if;
	end if;
      end if;
    end process;

    u_ddr4_ii : ddr4 
      port map (
        global_reset_n      => reset_n,
        pll_ref_clk         => MB_II_REF_CLK,
        oct_rzqin           => MB_II_RZQ,
        mem_ck              => mb_ii_ck,
        mem_ck_n            => mb_ii_ck_n,
        mem_a               => mb_ii_a_internal,
        mem_act_n           => mb_ii_act_n,
        mem_ba              => mb_ii_ba,
        mem_bg              => mb_ii_bg,
        mem_cke             => mb_ii_cke,
        mem_cs_n            => mb_ii_cs, 
        mem_odt             => mb_ii_odt,
        mem_reset_n         => mb_ii_reset_n,
        mem_alert_n         => mb_ii_alert_n,
        mem_par             => mb_ii_parity,
        mem_dqs             => mb_ii_dqs,
        mem_dqs_n           => mb_ii_dqs_n,
        mem_dq(63 downto 0) => mb_ii_dq,
        mem_dq(71 downto 64)=> mb_ii_cb,
        mem_dbi_n           => mb_ii_dm,
        local_cal_success   => local_ii_cal_success,
        local_cal_fail      => local_ii_cal_fail,
        emif_usr_reset_n    => local_ii_reset_n,
        emif_usr_clk        => local_ii_clk,
        amm_ready_0         => local_ii_ready,
        amm_read_0          => local_ii_read,
        amm_write_0         => local_ii_write,
        amm_address_0       => local_ii_address,
        amm_readdata_0      => local_ii_readdata,
        amm_writedata_0     => local_ii_writedata,
        amm_burstcount_0    => local_ii_burstcount,
        amm_byteenable_0    => local_ii_be,
        amm_readdatavalid_0 => local_ii_read_data_valid
     );


--    -- ****** Front side transceivers and diagnostics ******
--
    RING_0_TX <= tx_serial_data_front(47 downto 36);
    QSFP_0_TX  <= tx_serial_data_front(35 downto 32);
    QSFP_1_TX  <= tx_serial_data_front(31 downto 28);
    QSFP_2_TX  <= tx_serial_data_front(27 downto 24);
    QSFP_3_TX  <= tx_serial_data_front(23 downto 20);
    QSFP_4_TX  <= tx_serial_data_front(19 downto 16);
    QSFP_5_TX  <= tx_serial_data_front(15 downto 12);
    RING_1_TX <= tx_serial_data_front(11 downto 0);
    
    rx_serial_data_front <= RING_0_RX  
                          & QSFP_0_RX & QSFP_1_RX & QSFP_2_RX & QSFP_3_RX & QSFP_4_RX & QSFP_5_RX
                          & RING_1_RX ;

    gen_diagnostics_front : for i in 0 to 2 generate
      mms_diagnostics_front: ENTITY diagnostics_lib.mms_diagnostics
      GENERIC MAP(
        g_data_w       => c_xgmii_data_w,
        g_block_len    => c_block_len,
        g_nof_streams  => 16,
        g_separate_clk => FALSE
      )
      PORT MAP (
        mm_rst          => reset_p,
        mm_clk          => mm_clk,
        st_rst          => reset_p,
        st_clk          => mm_clk,
        mm_mosi         => reg_diagnostics_front_mosi(i),
        mm_miso         => reg_diagnostics_front_miso(i),
        src_out_arr     => diagnostics_front_src_out_arr((i+1)*16-1 downto i*16),
        src_in_arr      => diagnostics_front_src_in_arr((i+1)*16-1 downto i*16),
        snk_out_arr     => diagnostics_front_snk_out_arr((i+1)*16-1 downto i*16),
        snk_in_arr      => diagnostics_front_snk_in_arr((i+1)*16-1 downto i*16)
      );
    end generate gen_diagnostics_front;

    tr_10GbE_front: ENTITY tr_10GbE_lib.tr_10GbE
    GENERIC MAP(
      g_technology    => c_tech_arria10,
      g_sim           => false,
      g_sim_level     => 0,
      g_nof_macs      => 48,
      g_use_mdio      => false,
      g_mdio_epcs_dis => false,
      g_lpbk_sosi     => true,
      g_lpbk_xgmii    => false,
      g_lpbk_xaui     => false,
      g_use_hdr_ram   => true
    )
    PORT MAP (
      -- System
      mm_rst              => reset_p,
      mm_clk              => mm_clk,
      tr_clk              => SA_CLK,
      cal_rec_clk         => mm_clk,
      dp_rst              => reset_p,
      dp_clk              => mm_clk,
      -- MM registers
      reg_mac_mosi        => reg_mac_front_mosi,
      reg_mac_miso        => reg_mac_front_miso,
      reg_hdr_insert_mosi => reg_hdr_insert_front_mosi,
      ram_hdr_insert_mosi => ram_hdr_insert_front_mosi,
      ram_hdr_remove_mosi => ram_hdr_remove_front_mosi,
      ram_hdr_remove_miso => ram_hdr_remove_front_miso,
      src_out_arr         => diagnostics_front_snk_in_arr,
      src_in_arr          => diagnostics_front_snk_out_arr,
      snk_out_arr         => diagnostics_front_src_in_arr,
      snk_in_arr          => diagnostics_front_src_out_arr,
      xaui_tx_out_arr     => open,
      xaui_rx_in_arr      => xaui_rx_arr_dummy,
      tx_serial_data      => tx_serial_data_front,
      rx_serial_data      => rx_serial_data_front,
      mdio_mdc_arr        => open,
      mdio_mdat_in_arr    => (others => '0'),
      mdio_mdat_oen_arr   => open
    );



    -- ****** Back side transceivers ******

    BCK_TX <= tx_serial_data_back(47 downto 0);
    
    rx_serial_data_back <= BCK_RX;

    gen_diagnostics_back : for i in 0 to 2 generate
      mms_diagnostics_back : ENTITY diagnostics_lib.mms_diagnostics
      GENERIC MAP(
        g_data_w       => c_xgmii_data_w,
        g_block_len    => c_block_len,
        g_nof_streams  => 16,
        g_separate_clk => FALSE
      )
      PORT MAP (
        mm_rst          => reset_p,
        mm_clk          => mm_clk,
        st_rst          => reset_p,
        st_clk          => mm_clk,
        mm_mosi         => reg_diagnostics_back_mosi(i),
        mm_miso         => reg_diagnostics_back_miso(i),
        src_out_arr     => diagnostics_back_src_out_arr((i+1)*16-1 downto i*16),
        src_in_arr      => diagnostics_back_src_in_arr((i+1)*16-1 downto i*16),
        snk_out_arr     => diagnostics_back_snk_out_arr((i+1)*16-1 downto i*16),
        snk_in_arr      => diagnostics_back_snk_in_arr((i+1)*16-1 downto i*16)
      );
    end generate gen_diagnostics_back;

    tr_10GbE_back : ENTITY tr_10GbE_lib.tr_10GbE
    GENERIC MAP(
      g_technology    => c_tech_arria10,
      g_sim           => false,
      g_sim_level     => 0,
      g_nof_macs      => 48,
      g_use_mdio      => false,
      g_mdio_epcs_dis => false,
      g_lpbk_sosi     => false,
      g_lpbk_xgmii    => false,
      g_lpbk_xaui     => false,
      g_use_hdr_ram   => false
    )
    PORT MAP (
      -- System
      mm_rst              => reset_p,
      mm_clk              => mm_clk,
      tr_clk              => SB_CLK,
      cal_rec_clk         => mm_clk,
      dp_rst              => reset_p,
      dp_clk              => mm_clk,
      -- MM registers
      reg_mac_mosi        => reg_mac_back_mosi,
      reg_mac_miso        => reg_mac_back_miso,
      reg_hdr_insert_mosi => reg_hdr_insert_back_mosi,
      ram_hdr_insert_mosi => ram_hdr_insert_back_mosi,
      ram_hdr_remove_mosi => ram_hdr_remove_back_mosi,
      ram_hdr_remove_miso => ram_hdr_remove_back_miso,
      src_out_arr         => diagnostics_back_snk_in_arr,
      src_in_arr          => diagnostics_back_snk_out_arr,
      snk_out_arr         => diagnostics_back_src_in_arr,
      snk_in_arr          => diagnostics_back_src_out_arr,
      xaui_tx_out_arr     => open,
      xaui_rx_in_arr      => xaui_rx_arr_dummy,
      tx_serial_data      => tx_serial_data_back,
      rx_serial_data      => rx_serial_data_back,
      mdio_mdc_arr        => open,
      mdio_mdat_in_arr    => (others => '0'),
      mdio_mdat_oen_arr   => open
    );



    -- ****** node control for resets and wdi

    u_node_ctrl : entity unb_common_lib.unb_node_ctrl
      generic map (
        g_pulse_us => c_unb_tse_clk_freq / (10**6)   -- nof system clock cycles to get us period, equal to system clock frequency / 10**6
      )
      port map (
        xo_clk      => ETH_clk,
        xo_rst_n    => reset_n,
        sys_clk     => sys_clk,
        sys_locked  => sys_locked,
        sys_rst     => open,
        st_clk      => clk,
        st_rst      => open,
        wdi_in      => pout_wdi,
        wdi_out     => WDI,   -- overrule default WDI = 'Z' and let SW toggle WDI via pout_wdi to enable the watchdog
        pulse_us    => open,
        pulse_ms    => open,
        pulse_s     => open   -- could be used to toggle a LED
      );

    reset_p <= not reset_n;
  
    u_system_pll : system_pll
      port map(
        refclk       => ETH_CLK,
        rst          => reset_p,
        locked       => sys_locked,
        outclk_0     => mm_clk,  -- 100MHz
        outclk_1     => sys_clk, -- 300MHz
	outclk_2     => clk_125  -- 125MHz for 1ge
     );

    -- ****** i2c interfaces ******

    u_qsys : unb2_test_qsys
      port map (
        clk_clk                          => mm_clk,
        reset_reset_n                    => reset_n,
        avs_i2c_master_0_gs_sim_export   => cs_sim,
        avs_i2c_master_0_sync_export     => cs_sync,
        avs_i2c_master_0_i2c_scl_export  => sens_sc,
        avs_i2c_master_0_i2c_sda_export  => sens_sd,
        avs_i2c_master_1_gs_sim_export   => cs_sim,
        avs_i2c_master_1_sync_export     => cs_sync,
        avs_i2c_master_1_i2c_scl_export  => pmbus_sc,
        avs_i2c_master_1_i2c_sda_export  => pmbus_sd,
        avs_i2c_master_2_gs_sim_export   => cs_sim,
        avs_i2c_master_2_sync_export     => cs_sync,
        avs_i2c_master_2_i2c_scl_export  => bck_scl(0),
        avs_i2c_master_2_i2c_sda_export  => bck_sda(0),
        avs_i2c_master_3_gs_sim_export   => cs_sim,
        avs_i2c_master_3_sync_export     => cs_sync,
        avs_i2c_master_3_i2c_scl_export  => bck_scl(1),
        avs_i2c_master_3_i2c_sda_export  => bck_sda(1),
        avs_i2c_master_4_sync_export     => cs_sync,
        avs_i2c_master_4_gs_sim_export   => cs_sim,
        avs_i2c_master_4_i2c_scl_export  => bck_scl(2),
        avs_i2c_master_4_i2c_sda_export  => bck_sda(2),
        avs_i2c_master_5_sync_export     => cs_sync,
        avs_i2c_master_5_gs_sim_export   => cs_sim,
        avs_i2c_master_5_i2c_sda_export  => qsfp_sda(0),
        avs_i2c_master_5_i2c_scl_export  => qsfp_scl(0),
        avs_i2c_master_6_sync_export     => cs_sync,
        avs_i2c_master_6_gs_sim_export   => cs_sim,
        avs_i2c_master_6_i2c_sda_export  => qsfp_sda(1),
        avs_i2c_master_6_i2c_scl_export  => qsfp_scl(1),
        avs_i2c_master_7_sync_export     => cs_sync,
        avs_i2c_master_7_gs_sim_export   => cs_sim,
        avs_i2c_master_7_i2c_sda_export  => qsfp_sda(2),
        avs_i2c_master_7_i2c_scl_export  => qsfp_scl(2),
        avs_i2c_master_8_sync_export     => cs_sync,
        avs_i2c_master_8_gs_sim_export   => cs_sim,
        avs_i2c_master_8_i2c_sda_export  => qsfp_sda(3),
        avs_i2c_master_8_i2c_scl_export  => qsfp_scl(3),
        avs_i2c_master_9_sync_export     => cs_sync,
        avs_i2c_master_9_gs_sim_export   => cs_sim,
        avs_i2c_master_9_i2c_sda_export  => qsfp_sda(4),
        avs_i2c_master_9_i2c_scl_export  => qsfp_scl(4),
        avs_i2c_master_10_sync_export    => cs_sync,
        avs_i2c_master_10_gs_sim_export  => cs_sim,
        avs_i2c_master_10_i2c_sda_export => qsfp_sda(5),
        avs_i2c_master_10_i2c_scl_export => qsfp_scl(5),
        avs_i2c_master_11_sync_export    => cs_sync,
        avs_i2c_master_11_gs_sim_export  => cs_sim,
        avs_i2c_master_11_i2c_sda_export => mb_sda,
        avs_i2c_master_11_i2c_scl_export => mb_scl,
        eth_tse_0_serial_connection_rxp_0          => ETH_SGIN(0), 
        eth_tse_0_serial_connection_txp_0          => ETH_SGOUT(0), 
--        eth_tse_0_pcs_ref_clk_clock_connection_clk => clk_125,
        eth_tse_0_pcs_ref_clk_clock_connection_clk => ETH_CLK,
--        eth_tse_1_pcs_ref_clk_clock_connection_clk => clk_125,
        eth_tse_1_pcs_ref_clk_clock_connection_clk => ETH_CLK,
        eth_tse_1_serial_connection_rxp_0          => ETH_SGIN(1),   
        eth_tse_1_serial_connection_txp_0          => ETH_SGOUT(1),
        pio_0_external_connection_export           => ver_id_pmbusalert,
        reg_mac_front_read_export                  => reg_mac_front_mosi.rd,         
        reg_mac_front_address_export               => reg_mac_front_mosi.address(14 downto 0),     
        reg_mac_front_write_export                 => reg_mac_front_mosi.wr,      
        reg_mac_front_writedata_export             => reg_mac_front_mosi.wrdata(31 downto 0), 
        reg_mac_front_readdata_export              => reg_mac_front_miso.rddata(31 downto 0), 
        reg_mac_front_waitrequest_export           => reg_mac_front_miso.waitrequest,  
        reg_mac_back_address_export                => reg_mac_back_mosi.address(14 downto 0),    
        reg_mac_back_write_export                  => reg_mac_back_mosi.wr,     
        reg_mac_back_writedata_export              => reg_mac_back_mosi.wrdata(31 downto 0),
        reg_mac_back_read_export                   => reg_mac_back_mosi.rd,    
        reg_mac_back_readdata_export               => reg_mac_back_miso.rddata(31 downto 0),          
        reg_mac_back_waitrequest_export            => reg_mac_back_miso.waitrequest,      
        reg_hdr_insert_front_readdata_export       => open,
        reg_hdr_insert_front_read_export           => reg_hdr_insert_front_mosi.rd,   
        reg_hdr_insert_front_writedata_export      => reg_hdr_insert_front_mosi.wrdata(31 downto 0),   
        reg_hdr_insert_front_address_export        => reg_hdr_insert_front_mosi.address(2 downto 0),    
        reg_hdr_insert_front_write_export          => reg_hdr_insert_front_mosi.wr,     
        reg_hdr_insert_back_readdata_export        => open,  
        reg_hdr_insert_back_read_export            => reg_hdr_insert_back_mosi.rd,     
        reg_hdr_insert_back_writedata_export       => reg_hdr_insert_back_mosi.wrdata(31 downto 0),  
        reg_hdr_insert_back_write_export           => reg_hdr_insert_back_mosi.wr,     
        reg_hdr_insert_back_address_export         => reg_hdr_insert_back_mosi.address(2 downto 0),  
        ram_hdr_insert_front_readdata_export       => open,
        ram_hdr_insert_front_read_export           => ram_hdr_insert_front_mosi.rd,   
        ram_hdr_insert_front_writedata_export      => ram_hdr_insert_front_mosi.wrdata(31 downto 0),  
        ram_hdr_insert_front_write_export          => ram_hdr_insert_front_mosi.wr,     
        ram_hdr_insert_front_address_export        => ram_hdr_insert_front_mosi.address(5 downto 0),   
        ram_hdr_insert_back_read_export            => ram_hdr_insert_back_mosi.rd,   
        ram_hdr_insert_back_writedata_export       => ram_hdr_insert_back_mosi.wrdata(31 downto 0),  
        ram_hdr_insert_back_address_export         => ram_hdr_insert_back_mosi.address(5 downto 0),    
        ram_hdr_insert_back_write_export           => ram_hdr_insert_back_mosi.wr,      

        ram_hdr_remove_front_readdata_export       => ram_hdr_remove_front_miso.rddata(31 downto 0),    
        ram_hdr_remove_front_read_export           => ram_hdr_remove_front_mosi.rd,    
        ram_hdr_remove_front_writedata_export      => ram_hdr_remove_front_mosi.wrdata(31 downto 0), 
        ram_hdr_remove_front_write_export          => ram_hdr_remove_front_mosi.wr,     
        ram_hdr_remove_front_address_export        => ram_hdr_remove_front_mosi.address(5 downto 0),   
        ram_hdr_remove_back_address_export         => ram_hdr_remove_back_mosi.address(5 downto 0),    
        ram_hdr_remove_back_write_export           => ram_hdr_remove_back_mosi.wr,      
        ram_hdr_remove_back_writedata_export       => ram_hdr_remove_back_mosi.wrdata(31 downto 0),  
        ram_hdr_remove_back_read_export            => ram_hdr_remove_back_mosi.rd,       
        ram_hdr_remove_back_readdata_export        => ram_hdr_remove_back_miso.rddata(31 downto 0),   
        reg_diagnostics_front_0_address_export     => reg_diagnostics_front_mosi(0).address(5 downto 0),
        reg_diagnostics_front_0_write_export       => reg_diagnostics_front_mosi(0).wr,  
        reg_diagnostics_front_0_writedata_export   => reg_diagnostics_front_mosi(0).wrdata(31 downto 0),  
        reg_diagnostics_front_0_read_export        => reg_diagnostics_front_mosi(0).rd,       
        reg_diagnostics_front_0_readdata_export    => reg_diagnostics_front_miso(0).rddata(31 downto 0),   
        reg_diagnostics_front_1_readdata_export    => reg_diagnostics_front_miso(1).rddata(31 downto 0),   
        reg_diagnostics_front_1_read_export        => reg_diagnostics_front_mosi(1).rd,       
        reg_diagnostics_front_1_writedata_export   => reg_diagnostics_front_mosi(1).wrdata(31 downto 0),  
        reg_diagnostics_front_1_write_export       => reg_diagnostics_front_mosi(1).wr,      
        reg_diagnostics_front_1_address_export     => reg_diagnostics_front_mosi(1).address(5 downto 0),    
        reg_diagnostics_front_2_readdata_export    => reg_diagnostics_front_miso(2).rddata(31 downto 0),   
        reg_diagnostics_front_2_read_export        => reg_diagnostics_front_mosi(2).rd,       
        reg_diagnostics_front_2_writedata_export   => reg_diagnostics_front_mosi(2).wrdata(31 downto 0),  
        reg_diagnostics_front_2_write_export       => reg_diagnostics_front_mosi(2).wr,      
        reg_diagnostics_front_2_address_export     => reg_diagnostics_front_mosi(2).address(5 downto 0),    
        reg_diagnostics_back_0_readdata_export     => reg_diagnostics_back_miso(0).rddata(31 downto 0),    
        reg_diagnostics_back_0_read_export         => reg_diagnostics_back_mosi(0).rd,        
        reg_diagnostics_back_0_writedata_export    => reg_diagnostics_back_mosi(0).wrdata(31 downto 0),   
        reg_diagnostics_back_0_write_export        => reg_diagnostics_back_mosi(0).wr,       
        reg_diagnostics_back_0_address_export      => reg_diagnostics_back_mosi(0).address(5 downto 0),     
        reg_diagnostics_back_1_address_export      => reg_diagnostics_back_mosi(1).address(5 downto 0),     
        reg_diagnostics_back_1_write_export        => reg_diagnostics_back_mosi(1).wr,       
        reg_diagnostics_back_1_writedata_export    => reg_diagnostics_back_mosi(1).wrdata(31 downto 0),   
        reg_diagnostics_back_1_read_export         => reg_diagnostics_back_mosi(1).rd,        
        reg_diagnostics_back_1_readdata_export     => reg_diagnostics_back_miso(1).rddata(31 downto 0),   
        reg_diagnostics_back_2_address_export      => reg_diagnostics_back_mosi(2).address(5 downto 0),    
        reg_diagnostics_back_2_write_export        => reg_diagnostics_back_mosi(2).wr,      
        reg_diagnostics_back_2_writedata_export    => reg_diagnostics_back_mosi(2).wrdata(31 downto 0),  
        reg_diagnostics_back_2_read_export         => reg_diagnostics_back_mosi(2).rd,       
        reg_diagnostics_back_2_readdata_export     => reg_diagnostics_back_miso(2).rddata(31 downto 0)   
    );

-- bidirectional and misc
-- use PPS as output enable

    INTA <= inta_out when PPS = '1' else 'Z';
    INTB <= intb_out when PPS = '1' else 'Z';
    TESTIO <= testio_out when PPS = '1' else "ZZZZZZZZ";
    BCK_ERR <= bck_err_out when PPS = '1' else "ZZZ";

    inta_in <= INTA;
    intb_in <= INTB;
    testio_in <= TESTIO;
    bck_err_in <= BCK_ERR;
    
    inta_out <= intb_in;
    intb_out <= inta_in;
    testio_out(7 downto 4) <= testio_in(3 downto 0);
    testio_out(3 downto 0) <= testio_in(7 downto 4);
    bck_err_out(2) <= bck_err_in(1);
    bck_err_out(1) <= bck_err_in(0);
    bck_err_out(0) <= bck_err_in(2);


    ver_id_pmbusalert <= version & id & pmbus_alert;
    
end str;
