--------------------------------------------------------------------------------
--
-- Copyright (C) 2009
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------
 
LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE work.unb2_board_pkg.ALL;

-- Keep the UniBoard system info knowledge in this HDL entity and in the
-- corresponding software functions in unb_common.c,h. This avoids having to
-- define named constants for indexing the fields in the info word.

ENTITY unb2_board_system_info IS
  GENERIC (
    g_sim        : BOOLEAN := FALSE;
    g_fw_version : t_unb2_board_fw_version := c_unb2_board_fw_version;  -- firmware version x.y (4b.4b)
    g_aux        : t_c_unb2_board_aux := c_unb2_board_aux               -- aux contains the hardware version
  );
  PORT (
    clk         : IN  STD_LOGIC;
    hw_version  : IN  STD_LOGIC_VECTOR(g_aux.version_w-1 DOWNTO 0);
    id          : IN  STD_LOGIC_VECTOR(g_aux.id_w-1 DOWNTO 0);
    info        : OUT STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
    bck_id      : OUT STD_LOGIC_VECTOR(c_unb2_board_nof_uniboard_w-1 DOWNTO 0); -- ID[7:2]
    chip_id     : OUT STD_LOGIC_VECTOR(c_unb2_board_nof_chip_w-1 DOWNTO 0);     -- ID[1:0]
    node_id     : OUT STD_LOGIC_VECTOR(c_unb2_board_nof_node_w-1 DOWNTO 0);     -- ID[1:0]
    is_node2    : OUT STD_LOGIC   -- '1' for Node 2, else '0'.
  );
END unb2_board_system_info;


ARCHITECTURE str OF unb2_board_system_info IS

  SIGNAL cs_sim         : STD_LOGIC;
  
  SIGNAL hw_version_reg : STD_LOGIC_VECTOR(hw_version'RANGE);
  SIGNAL id_reg         : STD_LOGIC_VECTOR(id'RANGE);
  SIGNAL nxt_info       : STD_LOGIC_VECTOR(info'RANGE);

  SIGNAL nxt_bck_id     : STD_LOGIC_VECTOR(bck_id'RANGE);
  SIGNAL nxt_chip_id    : STD_LOGIC_VECTOR(chip_id'RANGE);
  SIGNAL nxt_node_id    : STD_LOGIC_VECTOR(node_id'RANGE);
  SIGNAL nxt_is_node2   : STD_LOGIC;
  
BEGIN

  p_reg : PROCESS(clk)
  BEGIN
    IF rising_edge(clk) THEN
      -- inputs
      hw_version_reg <= hw_version;
      id_reg         <= id;
      -- output
      info           <= nxt_info;
      bck_id         <= nxt_bck_id;
      chip_id        <= nxt_chip_id;
      node_id        <= nxt_node_id;
      is_node2       <= nxt_is_node2;
    END IF;
  END PROCESS;
  
  cs_sim <= is_true(g_sim);

  p_info : PROCESS(cs_sim, hw_version_reg, id_reg)
  BEGIN
    nxt_info               <= (OTHERS=>'0');
    nxt_info(23 DOWNTO 20) <= TO_UVEC(g_fw_version.hi, 4);
    nxt_info(19 DOWNTO 16) <= TO_UVEC(g_fw_version.lo, 4);
    nxt_info(10)           <= cs_sim;
    nxt_info(9 DOWNTO 8)   <= hw_version_reg;
    nxt_info(7 DOWNTO 0)   <= id_reg;
  END PROCESS;
 
  nxt_bck_id   <= id_reg(7 DOWNTO 2);
  nxt_chip_id  <= id_reg(1 DOWNTO 0);
  nxt_node_id  <= id_reg(1 DOWNTO 0);
  nxt_is_node2 <= '1' WHEN TO_UINT(id_reg(1 DOWNTO 0)) = 2 ELSE '0';
  
END str;
