-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Multi testbench for tr_10GbE the 10G Ethernet IO module
-- Description:
--   The multi-tb finishes when all instances have signaled tb_end.
-- Usage:
--   > as 5
--   > run -all

LIBRARY IEEE, technology_lib, tech_pll_lib, tech_mac_10g_lib;
USE IEEE.std_logic_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE tech_mac_10g_lib.tb_tech_mac_10g_pkg.ALL;
USE tech_pll_lib.tech_pll_component_pkg.ALL;


ENTITY tb_tb_tr_10GbE IS
END tb_tb_tr_10GbE;


ARCHITECTURE tb OF tb_tb_tr_10GbE IS
  CONSTANT c_644       : TIME := tech_pll_clk_644_period;
  CONSTANT c_156       : TIME := 6.4 ns;
  CONSTANT c_data_type : NATURAL := c_tb_tech_mac_10g_data_type_symbols;
  
  CONSTANT c_tb_end_vec : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS=>'1');
  SIGNAL   tb_end_vec   : STD_LOGIC_VECTOR(7 DOWNTO 0) := c_tb_end_vec;  -- sufficiently long to fit all tb instances
BEGIN

-- g_technology              : NATURAL := c_tech_select_default;
-- g_tb_end                  : BOOLEAN := TRUE;   -- when TRUE then tb_end ends this simulation, else a higher multi-testbench will end the simulation
-- g_no_dut                  : BOOLEAN := FALSE;  -- default FALSE to verify the DUT, else use TRUE to verify the tb itself without the DUT
-- g_dp_clk_period           : TIME :=  5 ns;     -- 200 MHz
-- g_sim_level               : NATURAL := 1;      -- 0 = use IP; 1 = use fast serdes model
-- g_nof_channels            : NATURAL := 1;
-- g_ref_clk_644_period      : TIME := tech_pll_clk_644_period;  -- for 10GBASE-R
-- g_ref_clk_156_period      : TIME := 6.4 ns;                   -- for XAUI
-- g_data_type               : NATURAL := c_tb_tech_mac_10g_data_type_symbols;
-- g_verify_link_recovery    : BOOLEAN := TRUE;
-- g_link_status_check       : STD_LOGIC_VECTOR(c_tech_mac_10g_link_status_w-1 DOWNTO 0) := "11"

  u_no_dut                     : ENTITY work.tb_tr_10GbE GENERIC MAP (c_tech_select_default, FALSE,  TRUE, 5   ns, 0, 1, c_644, c_156, c_data_type, TRUE, "11") PORT MAP (tb_end_vec(0));
  u_tr_10GbE                   : ENTITY work.tb_tr_10GbE GENERIC MAP (c_tech_select_default, FALSE, FALSE, 5   ns, 0, 1, c_644, c_156, c_data_type, TRUE, "11") PORT MAP (tb_end_vec(1));
  u_tr_10GbE_dp_clk_6_5ns      : ENTITY work.tb_tr_10GbE GENERIC MAP (c_tech_select_default, FALSE, FALSE, 6.5 ns, 0, 1, c_644, c_156, c_data_type, TRUE, "11") PORT MAP (tb_end_vec(2));
  u_tr_10GbE_nof_channels_is_2 : ENTITY work.tb_tr_10GbE GENERIC MAP (c_tech_select_default, FALSE, FALSE, 5   ns, 0, 2, c_644, c_156, c_data_type, TRUE, "11") PORT MAP (tb_end_vec(3));
  u_tr_10GbE_sim_level_is_1    : ENTITY work.tb_tr_10GbE GENERIC MAP (c_tech_select_default, FALSE, FALSE, 5   ns, 1, 1, c_644, c_156, c_data_type, TRUE, "11") PORT MAP (tb_end_vec(4));
  
  p_tb_end : PROCESS
  BEGIN
    WAIT UNTIL tb_end_vec=c_tb_end_vec;
    REPORT "Multi tb simulation finished." SEVERITY FAILURE;
    WAIT;
  END PROCESS;
END tb;
