-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Reusable test bench verification for the tech_mac_10g
-- Description:
-- . Verify that > 0 packets were transmitted and that all were received


LIBRARY IEEE, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE dp_lib.dp_stream_pkg.ALL;


ENTITY tb_tech_mac_10g_verify_rx_pkt_cnt IS
  GENERIC (
    g_nof_pkt     : NATURAL
  );
  PORT (
    tx_clk      : IN  STD_LOGIC;
    tx_sosi     : IN  t_dp_sosi;
    rx_clk      : IN  STD_LOGIC;
    rx_sosi     : IN  t_dp_sosi;
    tx_pkt_cnt  : OUT NATURAL;
    rx_pkt_cnt  : OUT NATURAL;
    rx_end      : IN  STD_LOGIC
  );
END tb_tech_mac_10g_verify_rx_pkt_cnt;


ARCHITECTURE tb OF tb_tech_mac_10g_verify_rx_pkt_cnt IS

  SIGNAL i_tx_pkt_cnt     : NATURAL := 0;
  SIGNAL i_rx_pkt_cnt     : NATURAL := 0;
  
BEGIN

  tx_pkt_cnt <= i_tx_pkt_cnt;
  rx_pkt_cnt <= i_rx_pkt_cnt;  

  -- Verification
  i_tx_pkt_cnt <= i_tx_pkt_cnt + 1 WHEN tx_sosi.sop='1' AND rising_edge(tx_clk);
  i_rx_pkt_cnt <= i_rx_pkt_cnt + 1 WHEN rx_sosi.eop='1' AND rising_edge(rx_clk);
  
  p_verify_pkt_cnt : PROCESS  
  BEGIN
    WAIT UNTIL rx_end='1';
    
    -- Verify that all transmitted packets have been received
    IF i_tx_pkt_cnt=0 THEN
      REPORT "No packets were transmitted." SEVERITY ERROR;
    ELSIF i_tx_pkt_cnt/=g_nof_pkt THEN
      REPORT "Not all packets were transmitted." SEVERITY ERROR;
    ELSIF i_rx_pkt_cnt=0 THEN
      REPORT "No packets were received." SEVERITY ERROR;
    ELSIF i_tx_pkt_cnt/=i_rx_pkt_cnt THEN
      REPORT "Not all transmitted packets were received." SEVERITY ERROR;
    END IF;
    
    WAIT;
  END PROCESS;
  
END tb;
